// Persistence of Vision Ray Tracer Scene Description File
// File: rain_fx.pov
// Vers: 3.6
// Desc: Example-Scene
// Date: 16.2.04 (dd.mm.yy)
// Auth: Tim Nikias Wenclawiak - www.NoLights.de
// Last Update: 22.10.06 (dd.mm.yy)

//Required Files
// parsys.inc - Tim's Particle System

//Description
// An example to create some more elaborate rain with an added effect:
// Rain falls from the sky and hits the various objects. Each object
// has its own effect: either create a couple of small droplets,
// or "vaporize" the particle, placing a small cloud instead.

//Commandline-Options: 25fps for 6 seconds:
//+KFI1 +KFF150

#default{finish{ambient 0}}
//Create new random-seed for every frame, used to randomly place the particles
#declare R=seed(frame_number);

global_settings{
 assumed_gamma 1
 max_trace_level 64
 max_intersections 256
 adc_bailout 1/128
 }

//The Particle-System
#include "parsys.inc"
//Particle-System Setup
#declare Animation_Time = 6;
#declare Parsys_Detail=<1/25,2,.3>;

//Excerpt from my Look_At-Macros
/******************************/
//Look_At
//Parameters:
// 1. Target-Position
//Returns:
// 1. Rotation-Vector
//Notes:
// Assumes object at <0,0,0> facing -z, y=sky_vector
#macro Look_At(Target)
 #if (Target.y!=0)
  #local X_Rot_Float = degrees(atan2(Target.y,vlength(Target*<1,0,1>))); #else
  #local X_Rot_Float = 0; #end
 #if (Target.x!=0)
  #local Y_Rot_Float = degrees(atan2(-Target.x,-Target.z)); #else
  #if (Target.z<0)
   #local Y_Rot_Float = 0; #else
   #local Y_Rot_Float =180; #end
  #end
 #local Result=<X_Rot_Float, Y_Rot_Float, 0>;
 (Result)
#end

//Environment
//===========
sky_sphere{pigment{gradient y color_map{[0 rgb 1][.4 rgb <.4,.4,1>][1 rgb z*.3]}scale 2 translate -1}}
camera{location <0,3.5,-6> look_at <0,1,0>}

#declare MainLight = <2.5,29,-2.5>;

light_source{
  MainLight rgb 1
  fade_power 2 fade_distance 26
  }
light_source{
  MainLight rgb <.02,.02,.12>
  shadowless
  }

#declare FillLight = .075;
light_source{
  <20,30,30> rgb FillLight
  shadowless
  }
light_source{
  <-30,2,0> rgb FillLight
  shadowless
  }

#declare Environment=
union{
  //Walls
  box{<-9,-1,-3.5><9,0,3.5>}
  box{<-9,0,3><9,3,3.5>}
  cylinder{<-9,3,3.2><9,3,3.2>,.2}
  
  //Large Platform
  union{
    //Main Plateau
    torus{.5,.1}
    torus{1,.1}
    union{
      cylinder{0,y*.3,.1}
      sphere{y*.3,.1}
      translate x*.85
    }
    //Protrusion Bars
    union{
      cylinder{0,y*.3,.1}
      sphere{y*.3,.1}
      translate x*-.85
    }
    union{
      cylinder{0,y*.3,.1}
      sphere{y*.3,.1}
      translate z*.85
    }
    union{
      cylinder{0,y*.3,.1}
      sphere{y*.3,.1}
      translate z*-.85
    }
    
    difference{
      cylinder{y*-.1,y*.1,1}
      cylinder{y*-.2,y*.2,.5}
    }
    rotate y*12
    rotate <-10,-24,0>
    translate <-1.5,1.2,0>
  }
  
  pigment{rgb 1}
  finish{
   specular .1 roughness .1
   brilliance 1.5
   }
  }

#declare Heat_Source =  
  //Large Platform
  union{
    light_source{<1.5,1.2,0> color rgb <1,.8,.2> fade_power 2 fade_distance 1.5 shadowless}
    //Main Plateau
    torus{.5,.1}
    torus{1,.1}
    union{
      cylinder{0,y*.3,.1}
      sphere{y*.3,.1}
      translate x*.85
    }
    //Protrusion Bars
    union{
      cylinder{0,y*.3,.1}
      sphere{y*.3,.1}
      translate x*-.85
    }
    union{
      cylinder{0,y*.3,.1}
      sphere{y*.3,.1}
      translate z*.85
    }
    union{
      cylinder{0,y*.3,.1}
      sphere{y*.3,.1}
      translate z*-.85
    }
    
    difference{
      cylinder{y*-.1,y*.1,1}
      cylinder{y*-.2,y*.2,.5}
    }
    rotate y*12
    rotate <-10,24,0>
    translate <1.5,1.2,0>
    pigment{rgb <1,.45,.1>}
    finish{
      ambient .8 diffuse 1
    }
  }

object{Environment}
object{Heat_Source}


//Define the particle-groups and macros
//=====================================
//Sets into how many smaller droplets a particle divides upon impact
#declare DivideInto = 5;
//Sets how many samples on the particle's surface to check for collisions
#declare Samples = 5;

//This Macro gets called when a raindrop hits a surface: it releases a couple new ones
#macro ShootDroplets(_Data,_ImpactTime,_NormalPos,_ImpactNormal)
  #local _Rot = Look_At(_ImpactNormal);
  #local _A=0;
  #while (_A<DivideInto)
  Particle_Add(
    "droplets",
    array[5]{
      <0,20,_Data[0].x+_Data[0].z>,
      <Samples,.05,0>,
      _Data[2],
      vrotate(vrotate(vrotate(<0,0,-1>,x*45*rand(R)),z*360*rand(R)),_Rot)*( min(vlength(_Data[3])-3,0)*rand(R)+3),
      <.4,.8,1>,
    }
  )
  #local _A=_A+1;
  #end
#end

//This Macro gets called when a raindrop hits a heated surface: it releases a vapor-particle
#macro ShootVapor(_Data,_ImpactTime,_NormalPos,_ImpactNormal)
  #local _Rot = Look_At(_ImpactNormal);
  Particle_Add(
    "vapor",
    array[5]{
      <0,20,_Data[0].x+_Data[0].z>,
      <0,.05,0>,
      _Data[2],
      vrotate(vrotate(vrotate(<0,0,-1>,x*45*rand(R)),z*360*rand(R)),_Rot)*( min(vlength(_Data[3])-3,0)*rand(R)+3),
      <0,0,.8>,
    }
  )
#end

//Visualisation Macro for the Raindrops
// Data[3] is the velocity of the particle, I stretch the particle
// along the Z-Axis a little depending on the velocity. Afterwards,
// I rotate the particle to align with the velocity's direction
// using my Look_At-Macro provided at the beginning of the scene.
// Note that these are blob-components, hence twice the size and a
// strength-value of 1: The blob threshold is set to .5, so a particle
// need double the radius to appear to be the correct size when
// bouncing off of objects.
#macro Rain_Particle(_Data)
  #local _Rot = Look_At(_Data[3]);
  #local _Stretch = min(vlength(_Data[3]),4)/4;
  sphere{0,_Data[1].y*2,1 //*2,1
    scale <1,1,1>+<-.4,-.4,.5>*_Stretch
    rotate _Rot
    translate _Data[2]
  }
#end

//Visualisation Macro for the Vapor-Particles
// Just a small sphere, growing in size and getting fainter
// as it grows.
#macro Vapor_Particle(Data)
  #local Age=Data[0].x;
  sphere{0,1 hollow
    pigment{rgbt 1}
    interior{
      media{
        method 2 intervals 5 samples 1,1
        emission max((1-Age)*2,0)*.25
        density{
          spherical color_map{[0 rgb 0][.65 rgb 1][1 rgb 1]}
          scale .8
          turbulence .24 lambda 2.5 omega .6 octaves 7
          rotate (Data[4]+Data[4]*Age)*360
        }
        density{
          bozo
          color_map{[0 rgb .7][1 rgb 1]}
          scale .5
          translate Data[2]*.5
        }
      }
    }
    scale 1*(Data[1].y+.3*min(Data[0].x,1)+.1*min(Data[0].x*10,1))
    translate Data[2]
  }
#end

//The main group "raindrop" receives the initial drops that fall into
// the scene. Upon hitting the different objects, different macros
// are called and the original particle destroyed. The different
// macros just place a couple of smaller droplets colored according
// to the object that was hit.
ParticleGroup_Begin("raindrop",<1/25,1/25,0>)
ParticleGroup_Gravity(9.81,<0,-1,0>)
ParticleGroup_Damping(.9)
ParticleGroup_Object("Environment",.75,1,2,"off","ShootDroplets")
ParticleGroup_Object("Heat_Source",.75,1,2,"off","ShootVapor")
ParticleGroup_End()

//The second group "droplets" handles the smaller droplets and simply
// calculates collisions with the environment and vaporization.
ParticleGroup_Begin("droplets",<1/25,1/25,0>)
ParticleGroup_Gravity(9.81,<0,-1,0>)
ParticleGroup_Damping(.75)
ParticleGroup_Object("Environment",.5,.95,0,"off","off")
ParticleGroup_Object("Heat_Source",.75,1,2,"off","ShootVapor")
ParticleGroup_End()

//The third group "vapor" handles drifting vapor, just moving
// along the environment by the wind..
ParticleGroup_Begin("vapor",<1/25,1/25,0>)
ParticleGroup_TurbWind(
  2,.6,9,.1, //Turbulence Settings
  <0,10,0>*clock, //Modifies position of field
  20, //Multiplier for the speeds generated by turbulence
  .1)
ParticleGroup_LinearWind(<0,1,0>,5,.2)
ParticleGroup_Object("Environment",0,0,0,"off","off")
ParticleGroup_Object("Heat_Source",0,0,0,"off","off")
ParticleGroup_End()


//Within the first 50 frames, the initial raindrops are placed, 5 per frame
#if (frame_number <= 50)
  #declare A=0;
  #while (A<5)
    Particle_Add(
      "raindrop",
      array[5]{
        <0,3,(clock+clock_delta*rand(R))*Animation_Time>,
        <Samples,.08,0>,
        <0,6,0>+<rand(R)*5-2.5,0,rand(R)*2.5-1.25>,
        <0,0,0>,
        <.4,.8,1>,
      }
    )
    #declare A=A+1;
  #end
#end

//Initiate and run the Particle-System
Parsys_Run()

//Debug some information
#debug concat("Frame ",str(frame_number,1,0)," with ",str(tnw_Parsys_ActiveGroupnumbers[0],1,0)," active particles...\n")

//Visualize the particles
blob{
  threshold .5
  Show_Particles("raindrop","Rain_Particle")
  Show_Particles("droplets","Rain_Particle")
  sphere{-y,.1,.1}
  pigment{rgb <.4,.4,1> transmit .9}
  interior{ior 1.44 fade_power 2 fade_distance .5}
  finish{
    specular .8 roughness .01
    reflection{0,1 fresnel}
    conserve_energy
    }
}

Show_Particles("vapor","Vapor_Particle")

